"""mangala-bulan: 1D Idealized Oxygen Diffusion Solver with Michaelis-Menten and Myoglobin"""
__version__ = "0.0.1"
__authors__ = ["Sandy H. S. Herho", "Gandhi Napitupulu"]
from .core.solver import OxygenDiffusionSolver
__all__ = ["OxygenDiffusionSolver"]
